@@ -55,16 +55,6 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 
-Classes
--------
-
-ColorCorrection
-    The base class for the ASC CDL, containing attributes for all ten of the
-    color conversion numbers needed to fully describe an ASC CDL.
-
-Functions
----------
-
 """
 
 # ==============================================================================
@@ -75,10 +65,11 @@ from __future__ import print_function
 
 from argparse import ArgumentParser
 from ast import literal_eval
+from xml.dom import minidom
 import os
 import re
 import sys
-import xml.etree.ElementTree as ElementTree
+from xml.etree import ElementTree
 
 # Python 3 compatibility
 try:
@@ -98,7 +89,7 @@ __author__ = "Sean Wallitsch"
 __copyright__ = "Copyright 2014, Sean Wallitsch"
 __credits__ = ["Sean Wallitsch", ]
 __license__ = "MIT"
-__version__ = "0.6"
+__version__ = "0.6.1"
 __maintainer__ = "Sean Wallitsch"
 __email__ = "shidarin@alphamatte.com"
 __status__ = "Development"
@@ -107,21 +98,6 @@ __status__ = "Development"
 # of the file, as they are dispatcher dictionaries that require the functions
 # to be parsed by python before the dictionary can be built.
 
-# Because it's getting late and I'm too tired to dive into writing XML today
-CC_XML = """<?xml version="1.0" encoding="UTF-8"?>
-<ColorCorrection id="{id}">
-    <SOPNode>
-        <Description></Description>
-        <Slope>{slopeR} {slopeG} {slopeB}</Slope>
-        <Offset>{offsetR} {offsetG} {offsetB}</Offset>
-        <Power>{powerR} {powerG} {powerB}</Power>
-    </SOPNode>
-    <SatNode>
-        <Saturation>{sat}</Saturation>
-    </SatNode>
-</ColorCorrection>
-"""
-
 if sys.version_info[0] >= 3:  # pragma: no cover
     enc = lambda x: bytes(x, 'UTF-8')  # pylint: disable=C0103
 else:  # pragma: no cover
@@ -143,6 +119,7 @@ HALT_ON_ERROR = False
 __all__ = [
     'AscColorSpaceBase',
     'AscDescBase',
+    'AscXMLBase',
     'ColorCollectionBase',
     'ColorCorrection',
     'ColorDecision',
@@ -166,6 +143,9 @@ __all__ = [
 class AscColorSpaceBase(object):  # pylint: disable=R0903
     """Base class for Asc XML type nodes that deal with colorspace
 
+    Description
+    ~~~~~~~~~~~
+
     This class is meant to be inherited by any node type that used viewing and
     input colorspace descriptions.
 
@@ -184,6 +164,16 @@ class AscColorSpaceBase(object):  # pylint: disable=R0903
             Viewing device, settings and environment. Individual
             :class:`ColorCorrections` can override this.
 
+    **Public Methods:**
+
+        parse_xml_input_desc()
+            Parses an ElementTree Element to find & add an InputDescription.
+            If none is found, ``input_desc`` will remain set to ``None``.
+
+        parse_xml_viewing_desc()
+            Parses an ElementTree Element to find & add a ViewingDescription.
+            If none is found, ``viewing_desc`` will remain set to ``None``.
+
     """
     def __init__(self):
         # For multiple inheritance support.
@@ -203,22 +193,24 @@ class AscColorSpaceBase(object):  # pylint: disable=R0903
                 found, set our ``input_desc``
 
         **Returns:**
-            None
+            (bool)
+                True if found InputDescription Fields (even if blank)
 
         **Raises:**
             None
 
         """
-        # If the text field is empty, this will return None, which is the default
-        # value of viewing_desc and input_desc anyway.
+        # If the text field is empty, this will return None, which is the
+        # default value of viewing_desc and input_desc anyway.
         try:
             self.input_desc = xml_element.find('InputDescription').text
+            return True
         except AttributeError:
-            pass
+            # We don't have an InputDescription and that's ok.
+            return False
 
     # =========================================================================
 
-
     def parse_xml_viewing_desc(self, xml_element):
         """Parses an ElementTree element to find & add a viewing description
 
@@ -228,18 +220,21 @@ class AscColorSpaceBase(object):  # pylint: disable=R0903
                 found, set our ``viewing_desc``
 
         **Returns:**
-            None
+            (bool)
+                True if found InputDescription Fields (even if blank)
 
         **Raises:**
             None
 
         """
-        # If the text field is empty, this will return None, which is the default
-        # value of viewing_desc and input_desc anyway.
+        # If the text field is empty, this will return None, which is the
+        # default value of viewing_desc and input_desc anyway.
         try:
             self.viewing_desc = xml_element.find('ViewingDescription').text
+            return True
         except AttributeError:
-            pass
+            # We don't have a ViewingDescription and that's ok.
+            return False
 
 # ==============================================================================
 
@@ -247,6 +242,9 @@ class AscColorSpaceBase(object):  # pylint: disable=R0903
 class AscDescBase(object):  # pylint: disable=R0903
     """Base class for most Asc XML type nodes, allows for infinite desc
 
+    Description
+    ~~~~~~~~~~~
+
     This class is meant to be inherited by any node type that uses description
     fields.
 
@@ -262,6 +260,12 @@ class AscDescBase(object):  # pylint: disable=R0903
             end of the list, but desc can also be replaced by passing it a list
             or tuple. Desc can be emptied by passing it None, [] or ().
 
+    **Public Methods:**
+
+        parse_xml_descs()
+            Parses an ElementTree Element for any Description tags and appends
+            any text they contain to the ``desc``.
+
     """
     def __init__(self):
         super(AscDescBase, self).__init__()
@@ -308,9 +312,81 @@ class AscDescBase(object):  # pylint: disable=R0903
 # ==============================================================================
 
 
-class ColorCollectionBase(AscDescBase, AscColorSpaceBase):  # pylint: disable=R0903
+class AscXMLBase(object):
+    """Base class for nodes which can be converted to XML Elements
+
+    Description
+    ~~~~~~~~~~~
+
+    This class contains several convenience attributes which can be used
+    to retrieve ElementTree Elements, or nicely formatted strings.
+
+    **Attributes:**
+
+        element : (<xml.etree.ElementTree.Element>)
+            etree style Element representing the node.
+
+        xml : (str)
+            A nicely formatted XML string representing the node.
+
+        xml_root : (str)
+            A nicely formatted XML, ready to write to file string representing
+            the node. Formatted as an XML root, it includes the xml version and
+            encoding tags on the first line.
+
+    **Public Methods:**
+
+        build_element()
+            A placeholder method to be overriden by inheriting classes, calling
+            it will always return None.
+
+    """
+    def __init__(self):
+        super(AscXMLBase, self).__init__()
+
+    # Properties ==============================================================
+
+    @property
+    def element(self):
+        """etree style Element representing the node."""
+        return self.build_element()
+
+    @property
+    def xml(self):
+        """A nicely formatted XML string representing the node"""
+        # We'll take the xml_root attrib, which is ready to write, and just
+        # remove the first line, which is the xml version and encoding.
+        dom_string = self.xml_root.split(enc('\n'))
+        return enc('\n').join(dom_string[1:])
+
+    @property
+    def xml_root(self):
+        """A nicely formatted XML string with a root element ready to write"""
+        xml_string = ElementTree.tostring(self.element, 'UTF-8')
+        dom_xml = minidom.parseString(xml_string)
+        dom_string = dom_xml.toprettyxml(indent="    ", encoding='UTF-8')
+        # Fix for ugly dom formatting prior to 2.7, taken from:
+        # http://stackoverflow.com/questions/749796/pretty-printing-xml-in-python
+        if sys.version_info[0] < 3 and sys.version_info[1] < 7:  # pragma: no cover pylint: disable=E0012
+            text_re = re.compile(r'>\n\s+([^<>\s].*?)\n\s+</', re.DOTALL)
+            dom_string = text_re.sub(r'>\g<1></', dom_string)
+        return dom_string
+
+    # Public Methods ==========================================================
+
+    def build_element(self):  # pragma: no cover pylint: disable=R0201
+        """Placeholder for reference by attributes. Will return None"""
+        return None
+
+# ==============================================================================
+
+
+class ColorCollectionBase(AscDescBase, AscColorSpaceBase, AscXMLBase):  # pylint: disable=R0903
     """Base class for ColorDecisionList and ColorCorrectionCollection.
 
+    Description
+    ~~~~~~~~~~~
+
     Collections need to store children and have access to descriptions,
     input descriptions, and viewing descriptions.
 
@@ -320,13 +396,63 @@ class ColorCollectionBase(AscDescBase, AscColorSpaceBase):  # pylint: disable=R0
 
     **Attributes:**
 
+        desc : [str]
+            Since all Asc nodes which can contain a single description, can
+            actually contain an infinite number of descriptions, the desc
+            attribute is a list, allowing us to store every single description
+            found during parsing.
+
+            Setting desc directly will cause the value given to append to the
+            end of the list, but desc can also be replaced by passing it a list
+            or tuple. Desc can be emptied by passing it None, [] or ().
+
+            Inherited from :class:`AscDescBase` .
+
+        element : (<xml.etree.ElementTree.Element>)
+            etree style Element representing the node. Inherited from
+            :class:`AscXMLBase` .
+
         input_desc : (str)
             Description of the color space, format and properties of the input
-            images. Individual :class:`ColorCorrections` can override this.
+            images. Inherited from :class:`AscColorSpaceBase` .
 
         viewing_desc : (str)
-            Viewing device, settings and environment. Individual
-            :class:`ColorCorrections` can override this.
+            Viewing device, settings and environment. Inherited from
+            :class:`AscColorSpaceBase` .
+
+        xml : (str)
+            A nicely formatted XML string representing the node. Inherited from
+            :class:`AscXMLBase`.
+
+        xml_root : (str)
+            A nicely formatted XML, ready to write to file string representing
+            the node. Formatted as an XML root, it includes the xml version and
+            encoding tags on the first line. Inherited from
+            :class:`AscXMLBase`.
+
+    **Public Methods:**
+
+        build_element()
+            Builds an ElementTree XML Element for this node and all nodes it
+            contains. ``element``, ``xml``, and ``xml_root`` attributes use
+            this to build the XML. This function is identical to calling the
+            ``element`` attribute. Overrides inherited placeholder method
+            from :class:`AscXMLBase` .
+
+        parse_xml_descs()
+            Parses an ElementTree Element for any Description tags and appends
+            any text they contain to the ``desc``. Inherited from
+            :class:`AscDescBase`
+
+        parse_xml_input_desc()
+            Parses an ElementTree Element to find & add an InputDescription.
+            If none is found, ``input_desc`` will remain set to ``None``.
+            Inherited from :class:`AscColorSpaceBase`
+
+        parse_xml_viewing_desc()
+            Parses an ElementTree Element to find & add a ViewingDescription.
+            If none is found, ``viewing_desc`` will remain set to ``None``.
+            Inherited from :class:`AscColorSpaceBase`
 
     """
     def __init__(self):
@@ -335,7 +461,7 @@ class ColorCollectionBase(AscDescBase, AscColorSpaceBase):  # pylint: disable=R0
 # ==============================================================================
 
 
-class ColorCorrection(AscDescBase, AscColorSpaceBase):  # pylint: disable=R0902
+class ColorCorrection(AscDescBase, AscColorSpaceBase, AscXMLBase):  # pylint: disable=R0902
     """The basic class for the ASC CDL
 
     Description
@@ -354,10 +480,6 @@ class ColorCorrection(AscDescBase, AscColorSpaceBase):  # pylint: disable=R0902
 
     Order of operations is Slope, Offset, Power, then Saturation.
 
-    Inherits desc attribute and setters from :class:`AscDescBase`
-
-    Inherits input_desc and viewing_desc from :class:`AscColorSpaceBase`
-
     **Class Attributes:**
 
         members : {str: :class`ColorCorrection`}
@@ -367,6 +489,22 @@ class ColorCorrection(AscDescBase, AscColorSpaceBase):  # pylint: disable=R0902
 
     **Attributes:**
 
+        desc : [str]
+            Since all Asc nodes which can contain a single description, can
+            actually contain an infinite number of descriptions, the desc
+            attribute is a list, allowing us to store every single description
+            found during parsing.
+
+            Setting desc directly will cause the value given to append to the
+            end of the list, but desc can also be replaced by passing it a list
+            or tuple. Desc can be emptied by passing it None, [] or ().
+
+            Inherited from :class:`AscDescBase` .
+
+        element : (<xml.etree.ElementTree.Element>)
+            etree style Element representing the node. Inherited from
+            :class:`AscXMLBase` .
+
         file_in : (str)
             Filepath used to create this CDL.
 
@@ -382,6 +520,10 @@ class ColorCorrection(AscDescBase, AscColorSpaceBase):  # pylint: disable=R0902
 
             Note that this shadows the builtin id.
 
+        input_desc : (str)
+            Description of the color space, format and properties of the input
+            images. Inherited from :class:`AscColorSpaceBase` .
+
         sat_node : ( :class:`SatNode` )
             Contains a reference to a single instance of :class:`SatNode` ,
             which contains the saturation value and descriptions.
@@ -390,6 +532,48 @@ class ColorCorrection(AscDescBase, AscColorSpaceBase):  # pylint: disable=R0902
             Contains a reference to a single instance of :class:`SopNode` ,
             which contains the slope, offset, power values and descriptions.
 
+        viewing_desc : (str)
+            Viewing device, settings and environment. Inherited from
+            :class:`AscColorSpaceBase` .
+
+        xml : (str)
+            A nicely formatted XML string representing the node. Inherited from
+            :class:`AscXMLBase`.
+
+        xml_root : (str)
+            A nicely formatted XML, ready to write to file string representing
+            the node. Formatted as an XML root, it includes the xml version and
+            encoding tags on the first line. Inherited from
+            :class:`AscXMLBase`.
+
+    **Public Methods:**
+
+        build_element()
+            Builds an ElementTree XML Element for this node and all nodes it
+            contains. ``element``, ``xml``, and ``xml_root`` attributes use
+            this to build the XML. This function is identical to calling the
+            ``element`` attribute. Overrides inherited placeholder method
+            from :class:`AscXMLBase` .
+
+        determine_dest()
+            When provided an output extension, determines the destination
+            filename to be written to based on ``file_in`` & ``id``.
+
+        parse_xml_descs()
+            Parses an ElementTree Element for any Description tags and appends
+            any text they contain to the ``desc``. Inherited from
+            :class:`AscDescBase`
+
+        parse_xml_input_desc()
+            Parses an ElementTree Element to find & add an InputDescription.
+            If none is found, ``input_desc`` will remain set to ``None``.
+            Inherited from :class:`AscColorSpaceBase`
+
+        parse_xml_viewing_desc()
+            Parses an ElementTree Element to find & add a ViewingDescription.
+            If none is found, ``viewing_desc`` will remain set to ``None``.
+            Inherited from :class:`AscColorSpaceBase`
+
     """
 
     members = {}
@@ -528,7 +712,29 @@ class ColorCorrection(AscDescBase, AscColorSpaceBase):  # pylint: disable=R0902
             # Register the new id with the dictionary
             ColorCorrection.members[self._id] = self
 
-    # Methods =================================================================
+    # Public Methods ==========================================================
+
+    def build_element(self):
+        """Builds an ElementTree XML element representing this CC"""
+        cc_xml = ElementTree.Element('ColorCorrection')
+        cc_xml.attrib = {'id': self.id}
+        if self.input_desc:
+            input_desc = ElementTree.SubElement(cc_xml, 'InputDescription')
+            input_desc.text = self.input_desc
+        if self.viewing_desc:
+            viewing_desc = ElementTree.SubElement(cc_xml, 'ViewingDescription')
+            viewing_desc.text = self.viewing_desc
+        for description in self.desc:
+            desc = ElementTree.SubElement(cc_xml, 'Description')
+            desc.text = description
+        if self.sop_node:
+            cc_xml.append(self.sop_node.element)
+        if self.sat_node:
+            cc_xml.append(self.sat_node.element)
+
+        return cc_xml
+
+    # =========================================================================
 
     def determine_dest(self, output):
         """Determines the destination file and sets it on the cdl"""
@@ -542,8 +748,40 @@ class ColorCorrection(AscDescBase, AscColorSpaceBase):  # pylint: disable=R0902
 # ==============================================================================
 
 
-class ColorDecision(AscDescBase, AscColorSpaceBase):  # pylint: disable=R0903
-    """Contains a media ref and a ColorCorrection or reference to CC"""
+class ColorDecision(AscXMLBase):  # pylint: disable=R0903
+    """Contains a media ref and a ColorCorrection or reference to CC.
+
+    Description
+    ~~~~~~~~~~~
+
+    This class is a stub for now, with no functionality.
+
+    **Attributes:**
+
+        element : (<xml.etree.ElementTree.Element>)
+            etree style Element representing the node. Inherited from
+            :class:`AscXMLBase` .
+
+        xml : (str)
+            A nicely formatted XML string representing the node. Inherited from
+            :class:`AscXMLBase`.
+
+        xml_root : (str)
+            A nicely formatted XML, ready to write to file string representing
+            the node. Formatted as an XML root, it includes the xml version and
+            encoding tags on the first line. Inherited from
+            :class:`AscXMLBase`.
+
+    **Public Methods:**
+
+        build_element()
+            Builds an ElementTree XML Element for this node and all nodes it
+            contains. ``element``, ``xml``, and ``xml_root`` attributes use
+            this to build the XML. This function is identical to calling the
+            ``element`` attribute. Overrides inherited placeholder method
+            from :class:`AscXMLBase` .
+
+    """
     def __init__(self):
         """Inits an instance of ColorDecision"""
         super(ColorDecision, self).__init__()
@@ -551,10 +789,62 @@ class ColorDecision(AscDescBase, AscColorSpaceBase):  # pylint: disable=R0903
 # ==============================================================================
 
 
-class ColorNodeBase(AscDescBase):  # pylint: disable=R0903
+class ColorNodeBase(AscDescBase, AscXMLBase):  # pylint: disable=R0903
     """Base class for SOP and SAT nodes.
 
-    Inherits desc from :class:`AscDescBase`
+    Description
+    ~~~~~~~~~~~
+
+    This class is meant only to be inherited by :class:`SopNode` and
+    :class:`SatNode` and should not be used outside of those classes.
+
+    It inherits from both :class:`AscDescBase` and :class:`AscXMLBase` giving
+    the child classes both ``desc`` and ``xml`` related functionality.
+
+    This class is also home to a private function which helps :class:`SopNode`
+    and :class:`SatNode` perform type and value checks on incoming values.
+
+    **Attributes:**
+
+        desc : [str]
+            Since all Asc nodes which can contain a single description, can
+            actually contain an infinite number of descriptions, the desc
+            attribute is a list, allowing us to store every single description
+            found during parsing.
+
+            Setting desc directly will cause the value given to append to the
+            end of the list, but desc can also be replaced by passing it a list
+            or tuple. Desc can be emptied by passing it None, [] or ().
+
+            Inherited from :class:`AscDescBase` .
+
+        element : (<xml.etree.ElementTree.Element>)
+            etree style Element representing the node. Inherited from
+            :class:`AscXMLBase` .
+
+        xml : (str)
+            A nicely formatted XML string representing the node. Inherited from
+            :class:`AscXMLBase`.
+
+        xml_root : (str)
+            A nicely formatted XML, ready to write to file string representing
+            the node. Formatted as an XML root, it includes the xml version and
+            encoding tags on the first line. Inherited from
+            :class:`AscXMLBase`.
+
+    **Public Methods:**
+
+        build_element()
+            Builds an ElementTree XML Element for this node and all nodes it
+            contains. ``element``, ``xml``, and ``xml_root`` attributes use
+            this to build the XML. This function is identical to calling the
+            ``element`` attribute. Overrides inherited placeholder method
+            from :class:`AscXMLBase` .
+
+        parse_xml_descs()
+            Parses an ElementTree Element for any Description tags and appends
+            any text they contain to the ``desc``. Inherited from
+            :class:`AscDescBase`
 
     """
     def __init__(self):
@@ -619,7 +909,7 @@ class ColorNodeBase(AscDescBase):  # pylint: disable=R0903
 # ==============================================================================
 
 
-class MediaRef(object):
+class MediaRef(AscXMLBase):
     """A directory of files or a single file used for grade reference
 
     Description
@@ -662,6 +952,10 @@ class MediaRef(object):
         directory : (str)
             The directory portion of the URI, without the protocol or filename.
 
+        element : (<xml.etree.ElementTree.Element>)
+            etree style Element representing the node. Inherited from
+            :class:`AscXMLBase` .
+
         exists : (bool)
             True if the path is present in the file system.
 
@@ -726,11 +1020,31 @@ class MediaRef(object):
             points to a directory, all sequences found in that directory will
             be in this list.
 
+        xml : (str)
+            A nicely formatted XML string representing the node. Inherited from
+            :class:`AscXMLBase`.
+
+        xml_root : (str)
+            A nicely formatted XML, ready to write to file string representing
+            the node. Formatted as an XML root, it includes the xml version and
+            encoding tags on the first line. Inherited from
+            :class:`AscXMLBase`.
+
+    **Public Methods:**
+
+        build_element()
+            Builds an ElementTree XML Element for this node and all nodes it
+            contains. ``element``, ``xml``, and ``xml_root`` attributes use
+            this to build the XML. This function is identical to calling the
+            ``element`` attribute. Overrides inherited placeholder method
+            from :class:`AscXMLBase` .
+
     """
 
     members = {}
 
     def __init__(self, ref_uri, parent=None):
+        super(MediaRef, self).__init__()
         self._protocol, self._dir, self._filename = self._split_uri(ref_uri)
         self.parent = parent
 
@@ -827,8 +1141,8 @@ class MediaRef(object):
             old_ref = self.ref
             self._protocol = value
             self._change_membership(old_ref=old_ref)
-            # We probably don't need to reset the cached properties, but we will
-            # just to be safe.
+            # We probably don't need to reset the cached properties, but we
+            # will just to be safe.
             self._reset_cached_properties()
         else:
             raise TypeError(
@@ -1013,6 +1327,22 @@ class SatNode(ColorNodeBase):
 
     **Attributes:**
 
+        desc : [str]
+            Since all Asc nodes which can contain a single description, can
+            actually contain an infinite number of descriptions, the desc
+            attribute is a list, allowing us to store every single description
+            found during parsing.
+
+            Setting desc directly will cause the value given to append to the
+            end of the list, but desc can also be replaced by passing it a list
+            or tuple. Desc can be emptied by passing it None, [] or ().
+
+            Inherited from :class:`AscDescBase` .
+
+        element : (<xml.etree.ElementTree.Element>)
+            etree style Element representing the node. Inherited from
+            :class:`AscXMLBase` .
+
         parent : ( :class:`ColorCorrection` )
             The parent :class:`ColorCorrection` instance that created this
             instance.
@@ -1024,6 +1354,30 @@ class SatNode(ColorNodeBase):
 
             sat can be set with a float, int or numeric string.
 
+        xml : (str)
+            A nicely formatted XML string representing the node. Inherited from
+            :class:`AscXMLBase`.
+
+        xml_root : (str)
+            A nicely formatted XML, ready to write to file string representing
+            the node. Formatted as an XML root, it includes the xml version and
+            encoding tags on the first line. Inherited from
+            :class:`AscXMLBase`.
+
+    **Public Methods:**
+
+        build_element()
+            Builds an ElementTree XML Element for this node and all nodes it
+            contains. ``element``, ``xml``, and ``xml_root`` attributes use
+            this to build the XML. This function is identical to calling the
+            ``element`` attribute. Overrides inherited placeholder method
+            from :class:`AscXMLBase` .
+
+        parse_xml_descs()
+            Parses an ElementTree Element for any Description tags and appends
+            any text they contain to the ``desc``. Inherited from
+            :class:`AscDescBase`
+
     """
 
     # XML Fields for SopNodes can be one of these names:
@@ -1067,13 +1421,28 @@ class SatNode(ColorNodeBase):
                 )
             )
 
+    # Public Methods ==========================================================
+
+    def build_element(self):
+        """Builds an ElementTree XML Element representing this SatNode"""
+        sat = ElementTree.Element('SATNode')
+        for description in self.desc:
+            desc = ElementTree.SubElement(sat, 'Description')
+            desc.text = description
+        op_node = ElementTree.SubElement(sat, 'Saturation')
+        op_node.text = _de_exponent(self.sat)
+        return sat
+
 # ==============================================================================
 
 
 class SopNode(ColorNodeBase):
     """Color node that contains slope, offset and power data.
 
-    slope, offset and saturation are stored internally as lists, but always
+    Description
+    ~~~~~~~~~~~
+
+    Slope, offset and saturation are stored internally as lists, but always
     returned as tuples to prevent index assignment from being successful. This
     protects the user from inadvertently setting a single value in the list
     to be a non-valid value, which might result in values not being floats or
@@ -1087,6 +1456,22 @@ class SopNode(ColorNodeBase):
 
     **Attributes:**
 
+        desc : [str]
+            Since all Asc nodes which can contain a single description, can
+            actually contain an infinite number of descriptions, the desc
+            attribute is a list, allowing us to store every single description
+            found during parsing.
+
+            Setting desc directly will cause the value given to append to the
+            end of the list, but desc can also be replaced by passing it a list
+            or tuple. Desc can be emptied by passing it None, [] or ().
+
+            Inherited from :class:`AscDescBase` .
+
+        element : (<xml.etree.ElementTree.Element>)
+            etree style Element representing the node. Inherited from
+            :class:`AscXMLBase` .
+
         parent : ( :class:`ColorCorrection` )
             The parent :class:`ColorCorrection` instance that created this
             instance.
@@ -1109,16 +1494,39 @@ class SopNode(ColorNodeBase):
             default: (0.0, 0.0, 0.0)
 
         power : (float, float, float)
-            An rgb tuple representing the power, which is the only function that
-            changes the response curve of the function. Note that this has the
-            opposite response to adjustments than a traditional gamma operator.
-            These values must be positive. If you set this attribute with a
-            single value, it will be copied over all 3 colors. Any single value
-            given can be a float, int or numeric string.
-
+            An rgb tuple representing the power, which is the only function
+            that changes the response curve of the function. Note that this has
+            the opposite response to adjustments than a traditional gamma
+            operator. These values must be positive. If you set this attribute
+            with a single value, it will be copied over all 3 colors. Any
+            single value given can be a float, int or numeric string.
 
             default: (1.0, 1.0, 1.0)
 
+        xml : (str)
+            A nicely formatted XML string representing the node. Inherited from
+            :class:`AscXMLBase`.
+
+        xml_root : (str)
+            A nicely formatted XML, ready to write to file string representing
+            the node. Formatted as an XML root, it includes the xml version and
+            encoding tags on the first line. Inherited from
+            :class:`AscXMLBase`.
+
+    **Public Methods:**
+
+        build_element()
+            Builds an ElementTree XML Element for this node and all nodes it
+            contains. ``element``, ``xml``, and ``xml_root`` attributes use
+            this to build the XML. This function is identical to calling the
+            ``element`` attribute. Overrides inherited placeholder method
+            from :class:`AscXMLBase` .
+
+        parse_xml_descs()
+            Parses an ElementTree Element for any Description tags and appends
+            any text they contain to the ``desc``. Inherited from
+            :class:`AscDescBase`
+
     """
 
     # XML Fields for SopNodes can be one of these names:
@@ -1276,8 +1684,8 @@ class SopNode(ColorNodeBase):
                 set_value = value
         else:
             raise TypeError(
-                '{name} cannot be set directly with objects of type: "{type}". '
-                'Value given: "{value}".'.format(
+                '{name} cannot be set directly with objects of type: "{type}".'
+                ' Value given: "{value}".'.format(
                     name=name.title(),
                     type=type(value),
                     value=value,
@@ -1286,11 +1694,59 @@ class SopNode(ColorNodeBase):
 
         return set_value
 
+    # Public Methods ==========================================================
+
+    def build_element(self):
+        """Builds an ElementTree XML Element representing this SopNode"""
+        sop = ElementTree.Element('SOPNode')
+        fields = ['Slope', 'Offset', 'Power']
+        for description in self.desc:
+            desc = ElementTree.SubElement(sop, 'Description')
+            desc.text = description
+        for i, grade in enumerate([self.slope, self.offset, self.power]):
+            op_node = ElementTree.SubElement(sop, fields[i])
+            op_node.text = '{valueR} {valueG} {valueB}'.format(
+                valueR=_de_exponent(grade[0]),
+                valueG=_de_exponent(grade[1]),
+                valueB=_de_exponent(grade[2])
+            )
+        return sop
+
 # ==============================================================================
 # PRIVATE FUNCTIONS
 # ==============================================================================
 
 
+def _de_exponent(notation):
+    """Translates scientific notation into float strings"""
+    notation = str(notation)
+    if 'e' not in notation:
+        return notation
+
+    notation = notation.split('e')
+    # Grab the exponent value
+    digits = int(notation[-1])
+    # Grab the value we'll be adding 0s to
+    value = notation[0]
+
+    if value.startswith('-'):
+        negative = '-'
+        value = value[1:]
+    else:
+        negative = ''
+
+    value = value.replace('.', '')
+
+    if digits < 0:
+        new_value = negative + '0.0' + '0' * (abs(digits) - 2) + value
+    else:
+        zeros = len(value)
+        new_value = negative + value + '0' * (abs(digits) - zeros) + '0.0'
+    return new_value
+
+# ==============================================================================
+
+
 def _sanitize(name):
     """Removes any characters in string name that aren't alnum or in '_.'"""
     if not name:
@@ -1744,23 +2200,8 @@ def parse_flex(edl_file):
 
 def write_cc(cdl):
     """Writes the ColorCorrection to a .cc file"""
-
-    xml = CC_XML.format(
-        id=cdl.id,
-        slopeR=cdl.slope[0],
-        slopeG=cdl.slope[1],
-        slopeB=cdl.slope[2],
-        offsetR=cdl.offset[0],
-        offsetG=cdl.offset[1],
-        offsetB=cdl.offset[2],
-        powerR=cdl.power[0],
-        powerG=cdl.power[1],
-        powerB=cdl.power[2],
-        sat=cdl.sat
-    )
-
     with open(cdl.file_out, 'wb') as cdl_f:
-        cdl_f.write(enc(xml))
+        cdl_f.write(cdl.xml_root)
 
 # ==============================================================================
 
@@ -1812,16 +2253,16 @@ def parse_args():
         "-i",
         "--input",
         help="specify the filetype to convert from. Use when CDLConvert "
-             "cannot determine the filetype automatically. Supported input "
-             "formats are: "
-             "{inputs}".format(inputs=str(INPUT_FORMATS.keys()))
+             "cannot determine the filetype automatically. Supported input "  # pylint: disable=C0330
+             "formats are: "  # pylint: disable=C0330
+             "{inputs}".format(inputs=str(INPUT_FORMATS.keys()))  # pylint: disable=C0330
     )
     parser.add_argument(
         "-o",
         "--output",
         help="specify the filetype to convert to, comma separated lists are "
-             "accepted. Defaults to a .cc XML. Supported output formats are: "
-             "{outputs}".format(outputs=str(OUTPUT_FORMATS.keys()))
+             "accepted. Defaults to a .cc XML. Supported output formats are: "  # pylint: disable=C0330
+             "{outputs}".format(outputs=str(OUTPUT_FORMATS.keys()))  # pylint: disable=C0330
     )
 
     args = parser.parse_args()
